"""
Copyright 2020 - 2023 Dennis Rohde

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
from .backend import *
from .stabbing import stabbing_path as _stabbing_path

config = Config()

def _optimize_centers(self, curves, consecutive_call=False):
    all_balls = self.compute_center_enclosing_balls(curves, False)
    for i, center_balls in enumerate(all_balls):
        path, _ = _stabbing_path(center_balls)
        self[i] = Curve(path, "center {}".format(i))

Clustering_Result.optimize_centers = _optimize_centers

def plot_curve(*curves, vertex_markings=True, savename=None, saveextension=None, return_fig=False, legend=True):
    import matplotlib.pyplot as plt
    from mpl_toolkits.mplot3d import Axes3D
    max_compl = 1
    max_dim = 1
    fig = plt.figure()
    ax = None
    for curve in curves:
        if isinstance(curve, backend.Curve):
            max_compl = max(max_compl, curve.complexity)
            max_dim = max(max_dim, curve.dimensions)
        elif isinstance(curve, backend.Curves):
            for curv in curve:
                max_compl = max(max_compl, curv.complexity)
                max_dim = max(max_dim, curv.dimensions)
        elif isinstance(curve, backend.Clustering_Result):
            for curv in curve:
                max_compl = max(max_compl, curv.complexity)
                max_dim = max(max_dim, curv.dimensions)
    if max_dim >= 3:
        ax = fig.gca(projection='3d')
    else:
        ax = fig.gca()
    for curve in curves:
        if isinstance(curve, backend.Curve):
            if curve.dimensions >= 3:
                p = ax.plot(curve.values[:, 0], curve.values[:, 1], curve.values[:, 2], linestyle='--', marker='o', label = curve.name, markersize = 7, markevery = curve.complexity)
                if vertex_markings:
                    ax.plot(curve.values[1:, 0], curve.values[1:, 1], curve.values[:, 2], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
            elif curve.dimensions == 2:
                p = ax.plot(curve.values[:, 0], curve.values[:, 1], linestyle='--', marker='o', label = curve.name, markersize = 7, markevery = curve.complexity)
                if vertex_markings:
                    ax.plot(curve.values[1:, 0], curve.values[1:, 1], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
            else:
                p = ax.plot([i * max_compl / len(curve) for i in range(len(curve))], curve.values, linestyle='--', marker='o', label = curve.name, markersize = 7, markevery = curve.complexity)
                if vertex_markings:
                    ax.plot([i * max_compl / len(curve) for i in range(1, len(curve))], curve.values[1:], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
        elif isinstance(curve, backend.Curves):
            for curv in curve:
                if curv.dimensions >= 3:
                    p = ax.plot(curv.values[:, 0], curv.values[:, 1], curv.values[:, 2], linestyle='--', marker='o', label = curv.name, markersize = 7, markevery = curv.complexity)
                    if vertex_markings:
                        ax.plot(curv.values[1:, 0], curv.values[1:, 1], curv.values[:, 2], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
                elif curv.dimensions == 2:
                    p = plt.plot(curv.values[:, 0], curv.values[:, 1], linestyle='--', marker='o', label = curv.name, markersize = 7, markevery = curv.complexity)
                    if vertex_markings:
                        plt.plot(curv.values[1:, 0], curv.values[1:, 1], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
                else:
                    p = plt.plot([i * max_compl / len(curv) for i in range(len(curv))], curv.values, linestyle='--', marker='o', label = curv.name, markersize = 7, markevery = curv.complexity)
                    if vertex_markings:
                        plt.plot([i * max_compl / len(curv) for i in range(1, len(curv))], curv.values[1:], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
        elif isinstance(curve, backend.Clustering_Result):
            for curv in curve:
                if curv.dimensions >= 3:
                    p = ax.plot(curv.values[:, 0], curv.values[:, 1], curv.values[:, 2], linestyle='-', marker='o', label = curv.name, markersize = 7, markevery = curv.complexity)
                    if vertex_markings:
                        ax.plot(curv.values[1:, 0], curv.values[1:, 1], curv.values[:, 2], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
                elif curv.dimensions == 2:
                    p = plt.plot(curv.values[:, 0], curv.values[:, 1], linestyle='-', marker='o', label = curv.name, markersize = 7, markevery = curv.complexity)
                    if vertex_markings:
                        plt.plot(curv.values[1:, 0], curv.values[1:, 1], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
                else:
                    p = plt.plot([i * max_compl / len(curv) for i in range(len(curv))], curv.values, linestyle='-', marker='o', label = curv.name, markersize = 7, markevery = curv.complexity)
                    if vertex_markings:
                        plt.plot([i * max_compl / len(curv) for i in range(1, len(curv))], curv.values[1:], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
    if legend:
        ax.legend(title='Curve names:')
    ax.set_title('Fred Curves')
    if savename is not None:
        plt.savefig("{}.{}".format(savename, saveextension), dpi=150)
        plt.close()
    elif return_fig:
        return fig
    else:
        plt.show()
        plt.close()
    
def plot_clustering(clustering_result, curves, vertex_markings=True, savename=None, saveextension=None, return_fig=False, legend=True):
    if not (isinstance(clustering_result, backend.Clustering_Result) and isinstance(curves, backend.Curves)):
        print("Check parameters!")
        return
    if len(clustering_result.assignment) < 1:
        print("compute_assignment was not called! calling now")
        clustering_result.compute_assignment(curves)
    from mpl_toolkits.mplot3d import Axes3D
    import matplotlib.pyplot as plt
    import matplotlib.colors as mcolors
    colors = list(mcolors.BASE_COLORS)
    if len(clustering_result) > len(colors):
        colors = list(mcolors.TABLEAU_COLORS)
    if len(clustering_result) > len(colors):
        colors = list(mcolors.mcolors.CSS4_COLORS)
    max_compl = 1
    max_dim = 1
    fig = plt.figure()
    ax = None
    for curve in curves:
        max_compl = max(max_compl, curve.complexity)
        max_dim = max(max_dim, curve.dimensions)
    if max_dim >= 3:
        ax = fig.gca(projection='3d')
    else:
        ax = fig.gca()
    for i, curve in enumerate(clustering_result):
        if curve.dimensions >= 3:
            p = ax.plot(curve.values[:, 0], curve.values[:, 1], curve.values[:, 2], linestyle='-', marker='o', label = curve.name, color = colors[i], markersize = 7, markevery = curve.complexity)
            if vertex_markings:
                ax.plot(curve.values[1:, 0], curve.values[1:, 1], curve.values[:, 2], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
        elif curve.dimensions == 2:
            p = ax.plot(curve.values[:, 0], curve.values[:, 1], linestyle='-', marker='o', label = curve.name, color = colors[i], markersize = 7, markevery = curve.complexity)
            if vertex_markings:
                ax.plot(curve.values[1:, 0], curve.values[1:, 1], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
        else:
            p = ax.plot([i * max_compl / len(curve) for i in range(len(curve))], curve.values, linestyle='-', marker='o', label = curve.name, color = colors[i], markersize = 7, markevery = curve.complexity)
            if vertex_markings:
                ax.plot([i * max_compl / len(curve) for i in range(1, len(curve))], curve.values[1:], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
    for i in range(len(clustering_result.assignment)):
        for j in range(clustering_result.assignment.count(i)):
            curve = curves[clustering_result.assignment.get(i,j)]
            if curve.dimensions >= 3:
                p = ax.plot(curve.values[:, 0], curve.values[:, 1], curve.values[:, 2], linestyle=':', marker='o', label = curve.name, color = colors[i], markersize = 7, markevery = curve.complexity)
                if vertex_markings:
                    ax.plot(curve.values[1:, 0], curve.values[1:, 1], curve.values[:, 2], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
            elif curve.dimensions == 2:
                p = ax.plot(curve.values[:, 0], curve.values[:, 1], linestyle=':', marker='o', label = curve.name, color = colors[i], markersize = 7, markevery = curve.complexity)
                if vertex_markings:
                    ax.plot(curve.values[1:, 0], curve.values[1:, 1], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
            else:
                p = ax.plot([i * max_compl / len(curve) for i in range(len(curve))], curve.values, linestyle=':', marker='o', label = curve.name, color = colors[i], markersize = 7, markevery = curve.complexity)
                if vertex_markings:
                    ax.plot([i * max_compl / len(curve) for i in range(1, len(curve))], curve.values[1:], linestyle='', marker='x', label = None, color = p[0].get_color(), markersize = 7)
    if legend:
        ax.legend(title='Curve names:')
    ax.set_title('Fred Clustering')
    if savename is not None:
        plt.savefig("{}.{}".format(savename, saveextension), dpi=150)
        plt.close()
    elif return_fig:
        return fig
    else:
        plt.show()
        plt.close()



"""
Copyright 2022 - 2023 Alexander Neuhaus, Dennis Rohde

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import numpy as np
import math
from scipy import optimize
import random

def rejection_sampling(dim, radius, center, n):
    points = []
    if radius == 0:
        points.append(center)
    else:
        for i in range(n):
            while True:
                p = np.random.uniform(low=0.0, high=1.0, size=dim) * 2 - 1
            
                if np.linalg.norm(p) <= 1:
                    break
            p *= radius
            p += center
            points.append(p)
    return np.array(points)

def unit_vector(vector):
    if np.linalg.norm(vector) == 0:
        return vector
    return vector / np.linalg.norm(vector)

def angle(v1, v2):
    v1_u = unit_vector(v1)
    v2_u = unit_vector(v2)

    return np.pi - abs(np.arccos(np.dot(v1_u, v2_u)))

def f(x, *args):
    if len(x) < 3:
        return -np.pi
    samples = args[0]
    start = args[1]
    end =  args[2]
    y = [int(xs) for xs in x]
    curve = list()
    vectors = list()
    angles = list()
    for i in range(start, end):
        curve.append(samples[i][y[i-start]])
    for i in range(len(curve)-1):
        vectors.append(curve[i+1] - curve[i])
    for i in range(len(vectors)-1):
        angles.append(-angle(vectors[i],vectors[i+1]))
    return max(angles)

def compute_stabber(samples, start, end):
    lb = list()
    ub = list()
    for i in range(start, end):
        lb.append(0)
        ub.append(len(samples[i])-0.001)

    res = optimize.dual_annealing(f, args=(samples, start, end), bounds=list(zip(lb,ub)))
    xf = res.x
    curve = list()
    for i in range(start, end):
        curve.append(samples[i][int(xf[i-start])])
    return(np.array(curve))

def outer_tangents(r1, c1, r2, c2):
    angle1 = math.atan2(c2[1] - c1[1], c2[0] - c1[0])
    if((c1 == c2).all()):
        angle2 = math.pi/2
    else:
        angle2 = math.acos(np.clip((r1 - r2) / np.linalg.norm(c1 - c2), -1, 1))

    t1StartX = c1[0] + math.cos(angle1 + angle2) * r1
    t1StartY = c1[1] + math.sin(angle1 + angle2) * r1
    t1EndX = c2[0] + math.cos(angle1 + angle2) * r2
    t1EndY = c2[1] + math.sin(angle1 + angle2) * r2

    t2StartX = c1[0] + np.cos(angle1 - angle2) * r1
    t2StartY = c1[1] + np.sin(angle1 - angle2) * r1
    t2EndX = c2[0] + np.cos(angle1 - angle2) * r2
    t2EndY = c2[1] + np.sin(angle1 - angle2) * r2

    t1 = [[t1StartX, t1StartY], [t1EndX, t1EndY]]
    t2 = [[t2StartX, t2StartY], [t2EndX, t2EndY]]
    return t1, t2
    
def convert_coordinates(p1, p2, p3):
    
    def angle(u, v):
        return np.arccos(np.dot(u,v)/(np.linalg.norm(u)*np.linalg.norm(v)))

    def rotate(p, i, ang):
        q = p.copy()
        q[i] = p[i] * np.cos(ang) - p[i + 1] * np.sin(ang)
        q[i + 1] = p[i] * np.sin(ang) + p[i + 1] * np.cos(ang)
        return q
        
    def ax_ang(p, i):
        u = np.zeros_like(p)
        u[i] = 1
        pp = np.zeros_like(p)
        pp[i] = p[i]
        pp[i + 1] = p[i + 1]
        if pp[i + 1] >= 0:
            return np.pi / 2 - angle(pp, u)
        else:
            return np.pi / 2 + angle(pp, u)
        
    def transform(y, z):
        for i in range(0, len(y) - 1):
            ang = ax_ang(y, i)
            y = rotate(y, i, ang)
            z = rotate(z, i, ang)
        return y, z
        
    def transform2(z):
        for i in range(0, len(z) - 2):
            z = rotate(z, i, ax_ang(z,i))
        return z
        
    x = np.zeros_like(p1)
    y = p2 - p1
    z = p3 - p1
    
    y, z = transform(y, z)
    z = transform2(z)
    
    return x[-2:], y[-2:], z[-2:]

def check_containment(c1, r1, c2, r2, p):
    
    def collinear(p1, p2, p3):
        x = np.linalg.norm(p1 - p2)
        y = np.linalg.norm(p2 - p3)
        z = np.linalg.norm(p3 - p1)
        d = [x, y, z]
        m = np.argmax(d)
        dd = [d[i] for i in range(3) if i != m]
        if round(d[m], 12) == round(sum(dd), 12):
            return True
        else:
            return False
        
    if collinear(c1, c2, p):
        if((np.linalg.norm(p - c1) <= r1) or (np.linalg.norm(p - c2) <= r2)):
            return True
        else:
            if((np.linalg.norm(p - c1) <= np.linalg.norm(c1 - c2))
               and (np.linalg.norm(p - c2) <= np.linalg.norm(c1 - c2))):
                return True
            return False
    
    p1, p2, p3 = convert_coordinates(c1, c2, p)
    t1, t2 = outer_tangents(r1, p1, r2, p2)
    t1A = t1[0]
    t1B = t1[1]
    t2A = t2[0]
    t2B = t2[1]
    
    p3p = np.array((p3[0], 0))
    
    if((np.linalg.norm(p - c1) <= r1) or (np.linalg.norm(p - c2) <= r2)):
        return True
    else:
        sign1 = (p3[0] - t1A[0]) * (t1B[1] - t1A[1]) - (p3[1] - t1A[1]) * (t1B[0] - t1A[0])
        sign2 = (p3[0] - t2A[0]) * (t2B[1] - t2A[1]) - (p3[1] - t2A[1]) * (t2B[0] - t2A[0])
        if((sign1 >= 0 and sign2 <= 0)
           and (np.linalg.norm(p3p - p1) <= np.linalg.norm(p1 - p2))
           and (np.linalg.norm(p3p - p2) <= np.linalg.norm(p1 - p2))):
            return True
        return False

def is_stabbable(balls, old_samples, new_samples, start, end):
    for j in range(start + 1, end):
        for i in range(start, j):
            c1 = balls[i][0]
            r1 = balls[i][1]
            c2 = balls[end][0]
            r2 = balls[end][1]
            points = [p for p in old_samples[j] if check_containment(c1.copy(), r1, c2.copy(), r2, p.copy())]
            new_samples[j] = np.array(points)
            if len(points) == 0:
                return False
    return True

def stabbing_path(balls, epsilon=0.1, n_samples=None):
    dim = len(balls[0][0])
    if n_samples is None:
        n_samples = int(10 * 1/epsilon * np.log(len(balls)))
    old_samples = list()
    new_samples = list()
    segments = list()
    for i in range(len(balls)):
        new_samples.append(rejection_sampling(dim, balls[i][1], balls[i][0], n_samples))
        old_samples.append(new_samples[i])
    start = 0;
    end = 0;
    stabbable = True
    while end < len(balls):
        if stabbable:            
            old_samples = new_samples.copy()
            stabbable = is_stabbable(balls, old_samples, new_samples, start, end)
            end += 1
        else:
            end -= 1
            tmp = compute_stabber(old_samples, start, end-1)
            segments.append((tmp[0],tmp[len(tmp)-1]))
            new_samples = old_samples.copy()
            start = end
            stabbable = True
    if not stabbable:
        tmp = compute_stabber(old_samples, start, end - 1)
        segments.append((tmp[0],tmp[len(tmp) - 1]))
        segments.append(np.array([old_samples[-1][0]]))
    else:
        tmp = compute_stabber(old_samples, start, end)
        segments.append((tmp[0],tmp[len(tmp)-1]))
        old_samples = new_samples
    return np.concatenate(segments), old_samples



"""
Copyright 2020 - 2023 Dennis Rohde

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
from . import backend

from cvxopt import matrix, solvers
solvers.options['show_progress'] = False
import numpy as np

def curve_within_radii(T, radii):
    l = len(T)
    start_point = _intersection_point(T, radii, [1] * l, [0] * l)
    if not start_point[0]:
        return (False, )
    end_point = _intersection_point(T, radii, [T[i].points - 1 for i in range(0, l)], [1] * l)
    if not end_point[0]:
        return (False, )
    borders = [([1] * len(T), start_point[1]), ([T[i].points for i in range(0, l)], end_point[1])]
    _compute_borders(T, radii, np.array([1] * l, dtype=np.uint64), borders)
    borders = sorted(borders, key=lambda border: border[0])
    solution = list()
    solution.append(borders[-1])
    alpha = [T[i].points for i in range(0, l)]
    for i in range(len(borders)-2, -1, -1):
        diff = np.array(alpha) - np.array(borders[i][0])
        bools = any(diff > 1) or any(np.array(borders[i][0]) > np.array(alpha)) or all(np.array(borders[i][0]) == np.array(alpha))
        if not bools:
            solution.append(borders[i])
            alpha = borders[i][0]
    if solution[-1] != borders[0]:
        return (False, )
    solution = reversed(solution)
    return (True, backend.Curve(np.array([sol[1] for sol in solution], dtype="double")))
    
def _compute_borders(T, radii, alpha, borders):
    for j in range(0, len(T)):
        if alpha[j] + 1 < T[j].points:
            border = _cellborder(T, radii, alpha, j, 1)
            if border[0]:
                mask = np.zeros((len(T), ))
                mask[j] = 1
                borders.append(([alpha[i] + border[1][0][i] for i in range(0, len(alpha))], border[1][1]))
                _compute_borders(T, radii, alpha + mask, borders)

def _intersection_point(T, radii, alpha, values):
    lines = list()
    for i in range(0, len(T)):
        lines.append([(1-values[i])*T[i][alpha[i]-1][k] + values[i]*T[i][alpha[i]][k] for k in range(0, T[i].dimensions)])
    
    c = matrix([0.] * T[0].dimensions)
    G = []
    for i in range(0, T[0].dimensions):
        m = len(T) * ([0.] + i * [0.] + [-1] + (T[0].dimensions - i - 1) * [0.])
        G.append(m)
        
    G = matrix(G)
    h = []
    for i in range(0, len(T)):
        h += [radii[i]] + [-lines[i][j] for j in range(0, T[i].dimensions)]
    h = matrix(h)
    dims = {'l': 0, 'q': [1+T[0].dimensions] * len(T), 's': [0]}
    sol = solvers.conelp(c, G, h, dims)
    return sol['status'] == 'optimal', np.array([s for s in sol['x']]) if sol['x'] is not None else None

def _cellborder(T, radii, alpha, j, value):
    lines = list()
    for i in range(0, len(T)):
        if (i == j): 
            lines.append(([(1-value)*T[i][int(alpha[i]-1)][k] + value*T[i][int(alpha[i])][k] for k in range(0, T[i].dimensions)],))
        else:
            lines.append(([T[i][int(alpha[i]-1)][k] - T[i][int(alpha[i])][k] for k in range(0, T[i].dimensions)], 
                          [T[i][int(alpha[i]-1)][k] for k in range(0, T[i].dimensions)]))
        
    dims = {'l': 2 * (len(T)-1), 'q': [1+T[0].dimensions] * len(T), 's': [0]}

    c = matrix([0.] * (len(T) - 1) + [0.] * T[0].dimensions)
    G = list()
    
    for i in range(0, len(T)):
        if i == j:
            pass
        elif i < j:
            m = i * [0.] + [1.] + (len(T) - i - 2) * [0.]
            m += i * [0.] + [-1.] + (len(T) - i - 2) * [0.]
            m += (i * (T[i].dimensions + 1)) * [0.] + [0.] + [-lines[i][0][k] for k in range(0, T[i].dimensions)] + ((len(T) - i - 1) * (T[i].dimensions + 1)) * [0.]
            G.append(m)
        else:
            m = (i - 1) * [0.] + [1.] + (len(T) - i - 1) * [0.]
            m += (i - 1) * [0.] + [-1.] + (len(T) - i - 1) * [0.]
            m += (i * (T[i].dimensions + 1)) * [0.] + [0.] + [-lines[i][0][k] for k in range(0, T[i].dimensions)] + ((len(T) - i - 1) * (T[i].dimensions + 1)) * [0.]
            G.append(m)
        
    for i in range(0, T[0].dimensions):
        m = (2 * (len(T) - 1)) * [0.]
        m += len(T) * ([0.] + i * [0.] + [-1] + (T[0].dimensions - i - 1) * [0.])
        G.append(m)
        
    G = matrix(G)
    
    h = [1.] * (len(T) - 1)
    h += [0.] * (len(T) - 1)
    for i in range(0, len(T)):
        h += [radii[i]]
        if i == j:
            h += [-lines[i][0][k] for k in range(0, T[i].dimensions)]
        else:
            h += [-lines[i][1][k] for k in range(0, T[i].dimensions)]
    h = matrix(h)
    try:
        sol = solvers.conelp(c, G, h, dims)
    except Exception as e:
        print(e)
    if sol['x'] is not None:
        x = sol['x']
        y = np.empty((len(T),))
        for i in range(0, len(T)):
            if i < j:
                y[i] = x[i]
            elif i == j:
                y[j] = value
            else:
                y[i] = x[i-1]
    return sol['status'] == 'optimal', (y, np.array([s for s in sol['x'][len(T)-1:]])) if sol['x'] is not None else None
